% This function returns the stability of given solutions
% together with the Floquets exponents

function [check,eigen,vectors] = stabT(sys,ddlstab,diaginfos,params,HH)

nbddl=diaginfos.nbddl;
nbddlstab=diaginfos.nbddlstab;
tolstab=params.tolstab;

nbptstroncon = length(ddlstab(:,1));

if HH == 0
    check = zeros(nbptstroncon,1);
    eigen = zeros(nbptstroncon,nbddl);
    
    for i=1:nbptstroncon
    
        DFX = JT(sys,ddlstab(i,:));
        [v,d] = eig(DFX);
        alf=diag(d);
            
        if any((real(alf) > 0+tolstab))
            stab = 0;
        else
            stab = 1;
        end
    check(i,1) = stab;
    eigen(i,:) = alf';
    vectors(i,:) = reshape(v,nbddl*nbddl,1);
    end
    
else

    check = zeros(nbptstroncon,1);
    eigen = zeros(nbptstroncon,nbddl);

    for i=1:nbptstroncon
    
        [rho, vect] = stabTT(sys,ddlstab(i,:),nbddl,nbddlstab,HH);
        alf = log(rho)/2/pi*ddlstab(i,end);
        
        if any((abs(rho) > 1+tolstab))
            stab = 0;
        else
            stab = 1;
        end
        
        check(i,1) = stab;
        eigen(i,:) = alf';
        vectors(i,:) = reshape(vect,nbddl*nbddl,1);
    end   
end

end


