% This function computes the Floquet multipliers
% through time integration (RK4) for stability analysis

function [rho, vect] = stabTT(sys,ddlstab,nbddl,nbddlstab,HH)

Nech = sys.nech;
TT = 2.*pi/ddlstab(end);

if ddlstab(end) < 0.01
    errordlg('Frequency very small : might be dealing with infinite period.','Singular case','on');
end

tt=linspace(0,TT,Nech);
% tt=linspace(0,1,Nech);
dt=tt(2)-tt(1);

Yn(:,:,1) = eye(nbddl);

for ii=1:length(tt)
    ti = tt(ii);
    Yn(:,:,ii+1) = rungekutta4(sys,Yn(:,:,ii),dt,ti,ddlstab,nbddlstab,HH);
    if any(isnan(Yn(:,:,ii+1)))
        error(['Monodromy matrix computation failed : NaN detected at step number ' num2str(ii) ' out of ' num2str(Nech)]);
    elseif any(isinf(Yn(:,:,ii+1)))
        error(['Monodromy matrix computation failed : Inf detected at step number ' num2str(ii) ' out of ' num2str(Nech)]);
    end
end

[v,d] = eig(Yn(:,:,end));
rho=diag(d);
[absrho,indsort]=sort(abs(rho),1,'descend');
rho=rho(indsort);
vect=v(:,indsort);
end



function Ynp1 = rungekutta4(sys,Yn,dt,ti,ddlstab,nbddlstab,HH)

k1=JT(sys,recomposition_Fourier(ti,ddlstab,nbddlstab,HH))*Yn;
k2=JT(sys,recomposition_Fourier(ti+dt/2,ddlstab,nbddlstab,HH))*(Yn+dt/2*k1);
k3=JT(sys,recomposition_Fourier(ti+dt/2,ddlstab,nbddlstab,HH))*(Yn+dt/2*k2);
k4=JT(sys,recomposition_Fourier(ti+dt,ddlstab,nbddlstab,HH))*(Yn+dt*k3);

Ynp1 = Yn+dt/6*(k1+2*k2+2*k3+k4);

end



function UUfinal = recomposition_Fourier(ti,ddlstab,backddl,HH) 

TT = 2.*pi/ddlstab(end);
UU = ddlstab(1:backddl);
for j=1:HH
        
    UU = UU + ddlstab(backddl*(2*j-1)+1:backddl*2*j)*cos(2*pi*j*ti/TT)+...
              ddlstab(2*j*backddl+1:(2*j+1)*backddl)*sin(2*pi*j*ti/TT);
end

UUfinal = [UU ddlstab(end-1)];

% Z=reshape(ddlstab(1:(2*HH+1)*backddl),backddl,[]);
% ZZ = Z(:,1) + Z(:,2:2:end)*cos((1:HH)'*2*pi*ti) + Z(:,3:2:end)*sin((1:HH)'*2*pi*ti);
% UUfinal = [ZZ ; ddlstab(end-1)];

end