% Computes the ANM series for a SYSTENSEUR object
% at the given point (vector), using the given tangent vector.

function [Ups,FpnlA] = calculeseries(systens,Uo,Utj)

global Q_tens

%% Matrice tangente systeme au point de depart
dRdU = get_dRdU(systens,Uo);

%% Vecteur tangent exact a la branche au point de depart
U1 = tangente(systens,dRdU,Uo);


if (U1'*Utj) <0
  U1 = - U1;
end

%% Matrice tangente de la MAN
K = [dRdU ; (get_chemin(systens).*U1)'];
%invK = inv(K);
[LK,UK,pK,qK] = lu(K,'vector');

[neq, ninc] = size(dRdU);

pmax = get_ordre(systens);

Ups = zeros(ninc,pmax+1);
nbelem=get_nbelem(systens);
dQdU = [];


%% 
% Point de départ
% 

Ups(:,1) = Uo;

%%                 
% Premier Ordre   
%                 

Ups(:,2) = U1;

%%                 
% Ordres suivants 
%                 

for p=2:pmax
%%% [dQdUp : version plus rapide, mais qui demande beaucoup de mémoire]
%%% 28/04/2011  S.K.

% Préparation du calcul pour l'ordre p
dQdUp=sparse([],[],[],ninc,neq,nbelem);
   for i=1:neq
        dQdUp(:,i) = (Q_tens(:,ninc*(i-1)+1:i*ninc) + Q_tens(:,ninc*(i-1)+1:i*ninc)')*Ups(:,p) ;
   end

dQdU = [dQdU dQdUp];

% Calcul du second membre
Fpnl = zeros(ninc,1);
    if mod(p,2)==0
        Fpnl = - [dQdU(:,(p/2-1)*neq+1:p/2*neq)'*Ups(:,p/2+1)/2 ; 0] ;
    end
    for r=1:floor((p-1)/2)
        Fpnl = Fpnl - [dQdU(:,(r-1)*neq+1:r*neq)'*Ups(:,p-r+1) ; 0] ;
    end
    
    Ups(qK,p+1) = UK\(LK\Fpnl(pK)) ;
% Pour p=2 on a Ups = [U0, U1, U2]

end


%% Second membre supplementaire
FpnlA = zeros(neq,1);  

% Préparation du calcul
dQdUp=sparse([],[],[],ninc,neq,nbelem);
for i=1:neq
    dQdUp(:,i) = (Q_tens(:,ninc*(i-1)+1:i*ninc) + Q_tens(:,ninc*(i-1)+1:i*ninc)')*Ups(:,p+1) ;
end
dQdU = [dQdU dQdUp];

if mod(p+1,2)==0
    FpnlA = - dQdU(:,((p+1)/2-1)*neq+1:(p+1)/2*neq)'*Ups(:,(p+1)/2+1)/2 ;
end
for r=1:floor(p/2)
    FpnlA = FpnlA - dQdU(:,(r-1)*neq+1:r*neq)'*Ups(:,p-r+2) ;
end

disp(['branch sign : ',num2str(sign(prod(diag(UK))))]);
