!     MEX-file / MATLAB function "L" for time-domain
!     orthogonal collocation system continuation of
!     periodic orbits with MANLAB

!     S. Karkar, B. Cochelin, C. Vergez
!     July 2010

!     The calling arguments must be : (the_object_itself, Ninter, Neq)

!     IMPORTANT :   This file is only a header to the global fortran source that
!                   will be generated via the "make" command inside your
!                   class '@XXXX' directory. It WILL NOT compile alone.


subroutine mexFunction(nlhs, plhs, nrhs, prhs)

    
    implicit none
    mwPointer plhs(*), prhs(*)
    mwPointer mxGetPr, mxCreateDoubleMatrix
    mwPointer u_pr, v_pr, Ninter_pr, Neq_pr, q_pr
    integer(4) nlhs, nrhs, Ninter, Neq, Ntot
    real(8) tmp


!     get the harmonic number H and the equation number Neq
    Ninter_pr = mxGetPr(prhs(4))
    call mxCopyPtrToReal8(Ninter_pr,tmp,1)
    Ninter = int(tmp)

    Neq_pr = mxGetPr(prhs(5))
    call mxCopyPtrToReal8(Neq_pr,tmp,1)
    Neq=int(tmp)

!   total system size
    Ntot = 3*Ninter*Neq + Neq + 2

!     get the input arguments u, and v pointers

    u_pr = mxGetPr(prhs(2))
    v_pr = mxGetPr(prhs(3))


!     Create matrix for the return argument.
    plhs(1) = mxCreateDoubleMatrix(Ntot-1,1,0)
!     Load the data into q_pr, which is the output to MATLAB
    q_pr = mxGetPr(plhs(1))


!     Call the computational subroutine with U and V pointers and H, Neq and N values
    call CALCUL_Q(u_pr,v_pr,q_pr,Ninter,Ntot)


    return
end subroutine mexFunction

subroutine CALCUL_Q(u_pr,v_pr,q_pr,Ninter,Ntot)
 
    use petits_operateurs
    implicit none
    integer(4) Ninter,Ntot
    real(8) U(Ntot),V(Ntot),Q(Ntot-1),pu(Neq),pv(Neq),puprime(Neq),P(4,3),Pprime(4,3),x(3),&
lambda,f,Y(3*Ninter+1,Neq),Y2(3*Ninter+1,Neq)
    integer(4) i,j,k,n
    mwPointer u_pr,v_pr,q_pr

    interface
        function pQ(pu,pv)
            use petits_operateurs
            real(8), intent(IN) :: pu(Neq),pv(Neq)
            real(8) pQ(Neq)
        end function pQ
        function pL1(pu)
            use petits_operateurs
            real(8), intent(IN) :: pu(Neq)
            real(8) pL1(Neq)
        end function pL1
        function pM(pu)
            use petits_operateurs
            real(8), intent(IN) :: pu(Neq)
            real(8) pM(Neq)
        end function pM
    end interface

    call mxCopyPtrToReal8(u_pr,U,Ntot)
    call mxCopyPtrToReal8(v_pr,V,Ntot)

 
    Q=0.0d0
  
! Definition of the collocation points :
!     x_i (i=1,3) is the ith Gauss point of order 3 on the interval [0,1]
    x=(/ (1-dsqrt(3d0/5d0))/2d0, 1d0/2d0, (1d0+dsqrt(3d0/5d0))/2d0 /)

! Values of the Lagrange polynomials P_k (k=1,4) on the collocation points
! x_i (i=1,3). The polynomial P_k is defined by :
!       P_k(x) = prod[(x-l/3), l=0:3, l!=k] / prod[(k/3-l/3), l=0:3, l!=k]

    P=0d0

    P(1,:) = (x(:)-1d0/3d0)*(x(:)-2d0/3d0)*(x(:)-1) / ((-1d0/3d0)*(-2d0/3d0)*(-1d0))
    P(2,:) = x(:)*(x(:)-2d0/3d0)*(x(:)-1d0) / (1d0/3d0*(-1d0/3d0)*(-2d0/3d0))
    P(3,:) = x(:)*(x(:)-1d0/3d0)*(x(:)-1d0) / (2d0/3d0*1d0/3d0*(-1d0/3d0))
    P(4,:) = x(:)*(x(:)-1d0/3d0)*(x(:)-2d0/3d0) / (1d0*2d0/3d0*1d0/3d0)

! Derivative of the above Pk(xi)

    Pprime=0d0

    Pprime(1,:) = ((x(:)-1d0/3d0)*(x(:)-2d0/3d0)+(x(:)-1d0/3d0)*(x(:)-1d0)+(x(:)-2d0/3d0)*(x(:)-1d0)) &
/ ((-1d0/3d0)*(-2d0/3d0)*(-1d0))
    Pprime(2,:) = (x(:)*(x(:)-2d0/3d0)+x(:)*(x(:)-1d0)+(x(:)-2d0/3d0)*(x(:)-1d0)) / (1d0/3d0*(-1d0/3d0)*(-2d0/3d0))
    Pprime(3,:) = (x(:)*(x(:)-1d0/3d0)+x(:)*(x(:)-1d0)+(x(:)-1d0/3d0)*(x(:)-1d0)) / (2d0/3d0*1d0/3d0*(-1d0/3d0))
    Pprime(4,:) = (x(:)*(x(:)-1d0/3d0)+x(:)*(x(:)-2d0/3d0)+(x(:)-1d0/3d0)*(x(:)-2d0/3d0)) / (1d0*2d0/3d0*1d0/3d0)

    Pprime = Pprime*Ninter   ! derivatives rescaling because of time rescaling on [0,1]

  !------------------------------------------------------------------------------
  !     Q(U,V) depends on pQ(.,.)   lambda pL1(.) and  f pM(.) 
  !------------------------------------------------------------------------------

    lambda = V(Ntot-1)
    f = V(Ntot)


! reshape U and V in a nice form
    Y=0d0
    Y2=0d0

    do n=1,3*Ninter+1
        Y(n,:) = U((n-1)*Neq+1:n*Neq)    ! the nth time sample is a Neq-size column-vector
        Y2(n,:) = V((n-1)*Neq+1:n*Neq)    ! the nth time sample is a Neq-size column-vector
    enddo

    do j=1,Ninter
        do i=1,3
            pu = (/ (dot_product(Y((j-1)*3+1:(j-1)*3+4,k),P(:,i)),k=1,Neq) /)
            pv = (/ (dot_product(Y2((j-1)*3+1:(j-1)*3+4,k),P(:,i)),k=1,Neq) /)
            puprime = (/ (dot_product(Y((j-1)*3+1:(j-1)*3+4,k),Pprime(:,i)),k=1,Neq) /)

            Q(((j-1)*3+i-1)*Neq+1:((j-1)*3+i)*Neq) = pQ(pu,pv) + pL1(pu)*lambda - pM(puprime)*f
        enddo
    enddo


!----------------------------------------------------
!   ---  nothing for the periodicity condition    ---
  
!------------------------------------------------------------------------------ 
!   ---    nothing for the  phase   equation        ---
!------------------------------------------------------------------------------


    call mxCopyReal8ToPtr(Q,q_pr,Ntot-1)
return 
end subroutine CALCUL_Q

