% plotbranchXYHBM   Branch plot for Manlab/HBM with user defined x and y axis
% 
%    Hand=plotbranchHBMXY(U,Ustab,X,Y,fig,[xlab]) plots vector Y as a
%    function of vector X with the stability state computed by 
%    the Manlab/HBM method.
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab and Ustab the stability data. 
%
%    X and Y are two vectors, with length(X)=length(Y)=size(U,2)
% 
%    fig is a figure/axis handle to plot the bargraph on.
%
%    (optional) [xlab] is a string containing the name of the user defined X
%    axis
%
%    It returns in Hand a column vector of handles to the plotted 
%    lineseries objects.
%
%    Example: Hand=plotbranchXYHBM(U,Ustab,NRJ,OMEGA,1)
%             plots vector OMEGA as a function of vector ENERGY. 
%
%    By O. Thomas / Oct. 2010
%%%  Last revision : S. Karkar / June 2014

function Hand=plotbranchXYHBM(U,Ustab,X,Y,fig)

if ~isempty(varargin)
    xlab=varargin{1};
else
    xlab='lambda';
end

pencolorML;
style = {':','-'};

% Activate axis
% =============

if floor(fig)==fig
  figure(fig)
  hold on
else
  axes(fig)
  hold on
end

% Plotting the branches
% =====================

%  0 1 1 1 1 1  1 0 0 0 0 0 0
%0 1 1 1 1 1 1  0 0 0 0 0 0
%  1 0 0 0 0 0 -1 0 0 0 0 0

Hand=[];
Handleg=[];
hold on

omega = U(end,:);
if size(Ustab,1)>1
  stab = Ustab(1,:);
else
  stab=ones(size(omega));
end
Ibif=find((stab(2:end)-stab(1:end-1))~=0);
Ibif=[1 Ibif size(U,2)];

for bb=1:(length(Ibif)-1)
    II=Ibif(bb):Ibif(bb+1);
    st=style{stab(Ibif(bb)+1)+1};
    hand=plot(X(II),Y(II),st,'Color',Col(1,:),'linewidth',1);
    Hand=[Hand;hand];
end
hand=plot(X(Ibif(2:end-1)),Y(Ibif(2:end-1)),'.','Color',Col(1,:));
Hand=[Hand;hand];

xlabel(xlab)

box on
  
