% calcbranchHBM   Branch computation for Manlab/HBM.
%    UU=calcbranchHBM(Neq,U,Idisp,Hdisp) computs the 
%    amplitude of the harmonics components of the solution computed by the
%    Manlab/HBM method.
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab and Ustab the stability data. 
% 
%    Idisp contains the indices of the entries of the initial physical 
%    unknown vector u(t) to be displayed and Hdisp the corresponding
%    harmonics number.
%
%    Neq is the number of equations of the initial system of unknown
%    u(t)
% 
%    It returns UU, a matrix of size (length(Idisp),size(U,2)), 
%    containing the values of the branch amplitude.
%
%    By O. Thomas / Oct. 2010

function UU=calcbranchHBM(Neq,U,Idisp,Hdisp)


% Computing the branches
% ======================

Nb=length(Idisp);

for ib=1:Nb
  idisp=Idisp(ib);
  hdisp=Hdisp(ib);
  if hdisp==0
    I0=idisp;
    Amp=U(I0,:);
  else
    Icos=Neq+idisp+(hdisp-1)*2*Neq;
    Amp=sqrt(U(Icos,:).^2+U(Icos+Neq,:).^2);
  end
  UU(ib,:)=Amp;
end
  
